//This sketch demonstrates that the TPL5010 is functioning correctly by:
//  1.  Showing that the TPL5010 can periodically awaken the sleeping pro mini, and 
//  2.  Showing that the TPL5010 will hardware reset the pro mini if it becomes unresponsive.

//This sketch borrows from Nick Gammon's Sketch J here:  https://www.gammon.com.au/forum/?id=11497

#include <avr/sleep.h>

bool alarmEvent=false;  //true iff pin D2 is triggered
bool wakeEvent=false;  //true iff pin D3 is triggered
bool state0=true;  //the CPU wakes up at regular intervals and then goes back to sleep.  No LED flashes.
bool state1=false;  //like state1, but the LED flashes every time the CPU wakes up.
bool state2=false;  //make the CPU become unresponsive to the TPL5010, thereby forcing the TPL5010 to reboot the Pro Mini.

void alarm ()
{
  // cancel sleep as a precaution
  sleep_disable();
  // precautionary while we do other stuff
  detachInterrupt (0);
  alarmEvent=true;
}  // end of wake

void wake ()
{
  // cancel sleep as a precaution
  sleep_disable();
  // precautionary while we do other stuff
  detachInterrupt (1);
  wakeEvent=true;
}  // end of wake

void blinkityBlink(uint8_t pulses, uint8_t repetitions, bool enabled) {
  if (enabled) {
    for (int x=0;x<repetitions;x++) {
      for (int i=0;i<pulses;i++) {
        digitalWrite(LED_BUILTIN,HIGH);
        delay(20);
        digitalWrite(LED_BUILTIN,LOW);
        delay(100);
      }    
      delay(500);
    }
  }
}

void soundTheAlarm() {
  long lastMicrosTime=0;  //timestamp in microseconds
  for (int x=0;x<1000;x++) {
    digitalWrite(8,HIGH);
    lastMicrosTime=micros();
    while ((micros()-lastMicrosTime)<300) {} //wait until 300 microseconds have passed
    digitalWrite(8,LOW);
    while ((micros()-lastMicrosTime)<x) {} //wait until another x microseconds have passed
  }
}

void makeRebootSound() {
  long lastMicrosTime=0;  //timestamp in microseconds
  for (int y=0;y<2;y++) {
  for (int x=1000;x>950;x--) {
    digitalWrite(8,HIGH);
    lastMicrosTime=micros();
    while ((micros()-lastMicrosTime)<300) {} //wait until 300 microseconds have passed
    digitalWrite(8,LOW);
    while ((micros()-lastMicrosTime)<x) {} //wait until another 50 microseconds have passed
  }
  delay(200);
  }
}

void makeClickSound() {
  //digitalWrite(8,HIGH);
  //delay(20);
  //digitalWrite(8,LOW);
  tone(8,2000,2);
  delay(20);
  tone(8,5000,2);
  delay(20);
  tone(8,10000,2);
  delay(20);
  tone(8,15000,2);
  delay(20);
  tone(8,20000,2);
}

void setup() {
  Serial.begin(115200);
  Serial.println("Starting...");
  pinMode(9,INPUT);
  pinMode(8,OUTPUT);  //BUZZER ON-OFF pin
  pinMode(7,OUTPUT);  //DONE pin on TPL5010
  //pinMode(6,INPUT);
  pinMode(LED_BUILTIN,OUTPUT);
  digitalWrite(7,LOW);  //assert DONE in response to reset.
  blinkityBlink(3,1,true);
  digitalWrite(7,HIGH);  //assert DONE in response to reset.
  delay(1);
  digitalWrite(7,LOW);  //assert DONE in response to reset.
  makeRebootSound();
}


void loop() {
  if (alarmEvent) {  //if a leak detected or if TEST button pressed.
    Serial.println("Alarm.");
    Serial.flush();

    if (digitalRead(9)) {//if TEST button is pressed
      Serial.println("TEST button pressed.");
      Serial.flush();
      makeClickSound();
      if (!state1) {
        blinkityBlink(5,2,true);
        state1=true;
      }
      else {// become unresponsive to demonstrate that TPL5010 will reboot the pro mini
        Serial.println("Become unresponsive, to demonstrate that external watchdog will force a hardware reboot...");
        Serial.flush();
        blinkityBlink(5,3,true);
        while (true) {};  //tight infinite loop prevents pro mini from issuing DONE's to TPL5010
      }
      //delay(100);
      //makeClickSound();

      //while (digitalRead(9)) {//as long as the TESt button is pressed
        //soundTheAlarm();
      //}
    }
    else {
      Serial.println("Leak detected!");
      Serial.flush();
      while (digitalRead(2)) {//for as long as a leak is detected...
        soundTheAlarm();
      }
    }

  }
  else {//must be a wake event
    if (state1) {
      digitalWrite(LED_BUILTIN,HIGH);  //blink on the built-in LED for roughly 20ms
    }
    digitalWrite(7,LOW);  //assert DONE in response to being awakened.
    delay(10);
    digitalWrite(7,HIGH);
    delay(10);
    digitalWrite(7,LOW);
    digitalWrite(LED_BUILTIN,LOW);
    Serial.print("time=");
    Serial.print(millis());
    Serial.println("ms: Awoken.");
    Serial.flush();
  
    //if (millis()>5000) {
      //Serial.println("Become unresponsive, to demonstrate that external watchdog will force a hardware reboot...");
      //while (true) {}  //invoke a tight infinite loop that will force the external watchdog to reboot the MCU
    //}
  }

  // disable ADC
  ADCSRA = 0;  
  
  set_sleep_mode (SLEEP_MODE_PWR_DOWN);  
  sleep_enable();

  // Do not interrupt before we go to sleep, or the
  // ISR will detach interrupts and we won't wake.
  noInterrupts ();
  
  // will be called when pin D2 goes low  
  attachInterrupt (1, wake, FALLING);  //pin D3
  attachInterrupt (0,alarm,RISING);  //pin D2
  alarmEvent=false;
  wakeEvent=false;  
  EIFR = bit (INTF1);  // clear flag for interrupt 1
  EIFR = bit (INTF0);  // clear flag for interrupt 0
  
  // turn off brown-out enable in software
  // BODS must be set to one and BODSE must be set to zero within four clock cycles
  MCUCR = bit (BODS) | bit (BODSE);
  // The BODS bit is automatically cleared after three clock cycles
  MCUCR = bit (BODS); 
  
  // We are guaranteed that the sleep_cpu call will be done
  // as the processor executes the next instruction after
  // interrupts are turned on.
  interrupts ();  // one cycle
  sleep_cpu ();   // one cycle

}
